@@ -21,8 +21,7 @@
 set -eux -o pipefail
 
 # Make sure the right golang version is installed based on Makefile
-WANT_GOLANG_VERSION=$(grep '^GO_VERSION' Makefile | awk '{ print $3 }')
-./hack/jenkins/installers/check_install_golang.sh $WANT_GOLANG_VERSION /usr/local
+./hack/jenkins/installers/check_install_golang.sh /usr/local
 
 make upload-preloaded-images-tar
 make clean
