@@ -47,48 +47,63 @@ var (
 	schema = map[string]update.Item{
 		".github/workflows/build.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/master.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/pr.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/docs.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/time-to-k8s.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
+			},
+		},
+		".github/workflows/leaderboard.yml": {
+			Replace: map[string]string{
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/translations.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
-		".github/workflows/update_k8s_versions.yml": {
+		".github/workflows/update-k8s-versions.yml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
+			},
+		},
+		".github/workflows/update-golang-version.yml": {
+			Replace: map[string]string{
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
+			},
+		},
+		".github/workflows/update-golint-version.yml": {
+			Replace: map[string]string{
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
+			},
+		},
+
+		".github/workflows/time-to-k8s-public-chart.yml": {
+			Replace: map[string]string{
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		".github/workflows/pr_verified.yaml": {
 			Replace: map[string]string{
-				`GO_VERSION: '.*`: `GO_VERSION: '{{.StableVersion}}'`,
-			},
-		},
-		".travis.yml": {
-			Replace: map[string]string{
-				`go:\n  - .*`: `go:{{printf "\n  - %s" .StableVersion}}`,
-				`go: .*`:      `go: {{.StableVersion}}`,
+				`GO_VERSION: .*`: `GO_VERSION: '{{.StableVersion}}'`,
 			},
 		},
 		"go.mod": {
@@ -96,14 +111,11 @@ var (
 				`(?m)^go .*`: `go {{.StableVersionMM}}`,
 			},
 		},
-		"hack/jenkins/common.sh": {
-			Replace: map[string]string{
-				`sudo \.\/installers\/check_install_golang\.sh \".*\" \"\/usr\/local\"`: `sudo ./installers/check_install_golang.sh "{{.StableVersion}}" "/usr/local"`,
-			},
-		},
 		"Makefile": {
 			Replace: map[string]string{
-				`GO_VERSION \?= .*`: `GO_VERSION ?= {{.StableVersion}}`,
+				// searching for 1.* so it does NOT match "KVM_GO_VERSION ?= $(GO_VERSION:.0=)" in the Makefile
+				`GO_VERSION \?= 1.*`:             `GO_VERSION ?= {{.StableVersion}}`,
+				`GO_K8S_VERSION_PREFIX \?= v1.*`: `GO_K8S_VERSION_PREFIX ?= {{.K8SVersion}}`,
 			},
 		},
 	}
@@ -118,6 +130,8 @@ var (
 type Data struct {
 	StableVersion   string `json:"stableVersion"`
 	StableVersionMM string `json:"stableVersionMM"` // go.mod wants go version in <major>.<minor> format
+	K8SVersion      string `json:"k8sVersion"`      // as of v1.23.0 Kubernetes uses k8s version in golang image name because: https://github.com/kubernetes/kubernetes/pull/103692#issuecomment-908659826
+
 }
 
 func main() {
@@ -126,28 +140,33 @@ func main() {
 	defer cancel()
 
 	// get Golang stable version
-	stable, stableMM, err := goVersions()
+	stable, stableMM, k8sVersion, err := goVersions()
 	if err != nil || stable == "" || stableMM == "" {
 		klog.Fatalf("Unable to get Golang stable version: %v", err)
 	}
-	data := Data{StableVersion: stable, StableVersionMM: stableMM}
+	data := Data{StableVersion: stable, StableVersionMM: stableMM, K8SVersion: k8sVersion}
 	klog.Infof("Golang stable version: %s", data.StableVersion)
 
 	update.Apply(ctx, schema, data, prBranchPrefix, prTitle, prIssue)
 }
 
 // goVersion returns Golang stable version.
-func goVersions() (stable, stableMM string, err error) {
-	resp, err := http.Get("https://golang.org/VERSION?m=text")
+func goVersions() (stable, stableMM, k8sVersion string, err error) {
+	// will update to the same image that kubernetes project uses
+	resp, err := http.Get("https://raw.githubusercontent.com/kubernetes/kubernetes/master/build/build-image/cross/VERSION")
 	if err != nil {
-		return "", "", err
+		return "", "", "", err
 	}
 	body, err := ioutil.ReadAll(resp.Body)
 	if err != nil {
-		return "", "", err
+		return "", "", "", err
 	}
-	stable = strings.TrimPrefix(string(body), "go")
+	// example response: v1.23.0-go1.17-buster.0
+	stable = string(body)
+	k8sVersion = strings.Split(stable, "-")[0]
+	stable = strings.Split(stable, "-")[1]
+	stable = strings.Replace(stable, "go", "", 1)
 	mmp := strings.SplitN(stable, ".", 3)
 	stableMM = strings.Join(mmp[0:2], ".") // <major>.<minor> version
-	return stable, stableMM, nil
+	return stable, stableMM, k8sVersion, nil
 }
