@@ -29,13 +29,14 @@ import (
 )
 
 type sshConn struct {
-	name    string
-	service string
-	cmd     *exec.Cmd
-	ports   []int
+	name       string
+	service    string
+	cmd        *exec.Cmd
+	ports      []int
+	activeConn bool
 }
 
-func createSSHConn(name, sshPort, sshKey string, svc *v1.Service) *sshConn {
+func createSSHConn(name, sshPort, sshKey string, resourcePorts []int32, resourceIP string, resourceName string) *sshConn {
 	// extract sshArgs
 	sshArgs := []string{
 		// TODO: document the options here
@@ -49,17 +50,17 @@ func createSSHConn(name, sshPort, sshKey string, svc *v1.Service) *sshConn {
 
 	askForSudo := false
 	var privilegedPorts []int32
-	for _, port := range svc.Spec.Ports {
+	for _, port := range resourcePorts {
 		arg := fmt.Sprintf(
 			"-L %d:%s:%d",
-			port.Port,
-			svc.Spec.ClusterIP,
-			port.Port,
+			port,
+			resourceIP,
+			port,
 		)
 
 		// check if any port is privileged
-		if port.Port < 1024 {
-			privilegedPorts = append(privilegedPorts, port.Port)
+		if port < 1024 {
+			privilegedPorts = append(privilegedPorts, port)
 			askForSudo = true
 		}
 
@@ -70,8 +71,8 @@ func createSSHConn(name, sshPort, sshKey string, svc *v1.Service) *sshConn {
 	if askForSudo && runtime.GOOS != "windows" {
 		out.Styled(
 			style.Warning,
-			"The service {{.service}} requires privileged ports to be exposed: {{.ports}}",
-			out.V{"service": svc.Name, "ports": fmt.Sprintf("%v", privilegedPorts)},
+			"The service/ingress {{.resource}} requires privileged ports to be exposed: {{.ports}}",
+			out.V{"resource": resourceName, "ports": fmt.Sprintf("%v", privilegedPorts)},
 		)
 
 		out.Styled(style.Permissions, "sudo permission will be asked for it.")
@@ -87,9 +88,10 @@ func createSSHConn(name, sshPort, sshKey string, svc *v1.Service) *sshConn {
 	cmd := exec.Command(command, sshArgs...)
 
 	return &sshConn{
-		name:    name,
-		service: svc.Name,
-		cmd:     cmd,
+		name:       name,
+		service:    resourceName,
+		cmd:        cmd,
+		activeConn: false,
 	}
 }
 
@@ -127,10 +129,11 @@ func createSSHConnWithRandomPorts(name, sshPort, sshKey string, svc *v1.Service)
 	cmd := exec.Command("ssh", sshArgs...)
 
 	return &sshConn{
-		name:    name,
-		service: svc.Name,
-		cmd:     cmd,
-		ports:   usedPorts,
+		name:       name,
+		service:    svc.Name,
+		cmd:        cmd,
+		ports:      usedPorts,
+		activeConn: false,
 	}, nil
 }
 
@@ -142,14 +145,22 @@ func (c *sshConn) startAndWait() error {
 		return err
 	}
 
+	c.activeConn = true
 	// we ignore wait error because the process will be killed
 	_ = c.cmd.Wait()
 
+	// Wait is finished for connection, mark false.
+	c.activeConn = false
+
 	return nil
 }
 
 func (c *sshConn) stop() error {
-	out.Step(style.Stopping, "Stopping tunnel for service {{.service}}.", out.V{"service": c.service})
-
-	return c.cmd.Process.Kill()
+	if c.activeConn {
+		c.activeConn = false
+		out.Step(style.Stopping, "Stopping tunnel for service {{.service}}.", out.V{"service": c.service})
+		return c.cmd.Process.Kill()
+	}
+	out.Step(style.Stopping, "Stopped tunnel for service {{.service}}.", out.V{"service": c.service})
+	return nil
 }
